//
//  'MonoView' image/video wallpaper for BeOS.
//  Copyright (C) 1999, 2000
//  Okada Jun (yun@be-in.org)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  MonoView is yet another wallpaper running on BeOS Intel and PPC,
//  originally designed and developed by Okada Jun in 1999.
//  http://www.be-in.org/~yun/
//


#include <algorithm>
#include <strstream>
#include <cstring>
#include <Node.h>
#include <NodeInfo.h>
#include <Path.h>
#include <Alert.h>
#include <Roster.h>
#include <AppFileInfo.h>
#include "MVDefs.h"
#include "MVWindow.h"
#include "MVImageView.h"
#include "MVVideoView.h"
#include "MVApplication.h"


MVApplication::MVApplication(void)
  : BApplication(MV_SIGNATURE)
  , mWinCount(0)
{
  this->InitAboutContents();
}


MVApplication::~MVApplication(void)
{
}


void
MVApplication::RefsReceived(BMessage *msg)
{
  entry_ref ref;
  int i;

  for (i = 0; msg->FindRef("refs", i, &ref) == B_OK; i++)
    this->OpenWindow(ref);
}


void
MVApplication::ArgvReceived(int32 argc, char **argv)
{
  const char *cwd;
  BPath path;
  entry_ref ref;
  int i;

  cwd = this->CurrentMessage()->FindString("cwd");
  if (!cwd)
    return;

  for (i = 1; i < argc; i++)
    {
      if (argv[i][0] == '/')
	{
	  path.SetTo(argv[i]);
	}
      else
	{
	  path.SetTo(cwd);
	  path.Append(argv[i]);
	}

      if (path.InitCheck() != B_NO_ERROR)
	continue;

      get_ref_for_path(path.Path(), &ref);
      this->OpenWindow(ref);
    }
}


void
MVApplication::ReadyToRun(void)
{
  if (mWin.empty())
    this->PostMessage(B_QUIT_REQUESTED);
}


void
MVApplication::MessageReceived(BMessage *msg)
{
  switch (msg->what)
    {
    case MV_WIN_CLOSED:
      {
	entry_ref ref;
	int i;
	
	for (i = 0; msg->FindRef("refs", i, &ref) == B_OK; i++)
	  this->WindowClosed(ref);
	break;
      }

    case MV_ACTIVATE_WINDOW:
      this->ActivateWindow(msg->FindInt32("index"));
      break;

    case MV_NEXT_WINDOW:
      this->NextWindow();
      break;

    case MV_PREV_WINDOW:
      this->PrevWindow();
      break;

    default:
      BApplication::MessageReceived(msg);
    }
}


void
MVApplication::OpenWindow(const entry_ref &ref)
{
  // check if file was already opened.
  if (mWin.find(ref) != mWin.end())
    {
      mWin[ref]->Activate();
      return;
    }

  // check file type.
  BNode node(&ref);
  if (node.InitCheck() != B_OK)
    return;
  BNodeInfo info(&node);
  if (info.InitCheck() != B_OK)
    return;

  char mime_type[B_MIME_TYPE_LENGTH];
  info.GetType(mime_type);

  MVControlView *view;
  if (strncmp(mime_type, "image/", 6) == 0)
    view = new MVImageView(ref);
  else if (strncmp(mime_type, "video/", 6) == 0)
    view = new MVVideoView(ref);
  else
    return;

  if (view->InitCheck() != B_OK)
    {
      delete view;
      return;
    }

  view->SetAboutContents(mAboutContents);

  BWindow *win = new MVWindow(view->Bounds().
			      OffsetByCopy
			      ((mWinCount % 10) * 20 + 50,
			       (mWinCount % 10) * 20 + 50),
			      ref);
  win->AddChild(view);
  win->Show();
  mWinCount++;

  mWin[ref] = win;
  mWinList.push_back(win);
}


void
MVApplication::WindowClosed(const entry_ref &ref)
{
  map<entry_ref, BWindow*>::iterator i;

  i = mWin.find(ref);
  if (i == mWin.end())
    return;

  mWinList.erase(find(mWinList.begin(), mWinList.end(), (*i).second));
  mWin.erase(i);

  if (mWin.empty())
    this->Quit();
}


int
MVApplication::ActiveWindow(void)
{
  vector<BWindow*>::iterator i;

  for (i = mWinList.begin(); i != mWinList.end(); i++)
    if ((*i)->IsActive())
      return i - mWinList.begin();

  return 0;
}


void
MVApplication::ActivateWindow(int index)
{
  if (index < mWinList.size())
    mWinList[index]->Activate();
}
    

void
MVApplication::NextWindow(void)
{
  mWinList[(this->ActiveWindow() + 1) % mWinList.size()]
    ->Activate();
}


void
MVApplication::PrevWindow(void)
{
  mWinList[(this->ActiveWindow() + mWinList.size() -  1) % mWinList.size()]
    ->Activate();
}


void
MVApplication::GetWindowList(vector<BWindow *> &outList)
{
  this->Lock();

  outList = mWinList;

  this->Unlock();
}


void
MVApplication::InitAboutContents(void)
{
  strstream about;
  app_info app_info;
  BFile file;
  BAppFileInfo app_file_info;
  version_info ver_info;
  char *s;
  char buf[B_MIME_TYPE_LENGTH];

  this->GetAppInfo(&app_info);
  file.SetTo(&app_info.ref, B_READ_WRITE);
  app_file_info.SetTo(&file);
  app_file_info.GetVersionInfo(&ver_info, B_APP_VERSION_KIND);

  about << app_info.ref.name
	<< ver_info.major << '.'
	<< ver_info.middle << '.'
	<< ver_info.minor
	<< "dabggf"[ver_info.variety] << ver_info.internal << "\n"
	<< ver_info.long_info
	<< '\0';

  mAboutContents = about.str();
}


void
MVApplication::AboutRequested(void)
{
  BAlert *aboutDialog = new BAlert("About", mAboutContents.c_str(), "Ok");
  aboutDialog->Go();
}


int
main(int argc, char **argv)
{
  MVApplication app;

  app.Run();

  return 0;
}


