//
//  'MonoView' image/video wallpaper for BeOS.
//  Copyright (C) 1999, 2000
//  Okada Jun (yun@be-in.org)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  MonoView is yet another wallpaper running on BeOS Intel and PPC,
//  originally designed and developed by Okada Jun in 1999.
//  http://www.be-in.org/~yun/
//


#include <algorithm>

#include <Bitmap.h>
#include <Screen.h>
#include <Window.h>
#include <Application.h>

#include "MVDefs.h"
#include "MVPopUpMenu.h"
#include "MVControlView.h"
#include "MVControlApplicationState.h"


MVControlApplicationState*
MVControlApplicationState::Instance(void)
{
  static MVControlApplicationState *instance;

  if (!instance)
    instance = new MVControlApplicationState();

  return instance;
}


MVControlApplicationState::MVControlApplicationState(void)
{
}


void
MVControlApplicationState::AllAttached(MVControlView *view)
{
  MVControlState::AllAttached(view);

  view->MakeFocus();
  view->FitToScreen();
  view->InscribedToFrame();
  view->SetTitle();
}


void
MVControlApplicationState::KeyDown(MVControlView *view, 
				   const char *bytes, int32 numBytes)
{
  if (numBytes != 1)
    return;

  int32 modifiers;
  view->Looper()->CurrentMessage()->FindInt32("modifiers", &modifiers);

  switch (bytes[0])
    {
    case '+':
      this->ZoomIn(view, modifiers);
      break;

    case '-':
      this->ZoomOut(view, modifiers);
      break;

    case '/':
      this->Normalize(view);
      break;

    case '*':
      this->FitToScreen(view);
      break;

    case '0':  case '1':  case '2':  case '3':  case '4':
    case '5':  case '6':  case '7':  case '8':  case '9':
      this->ActivateWindow(view, bytes[0] - '0');
      break;

    case '.':
    case 'p':
      this->PreviousWindow(view);
      break;

    case '\n':
    case 'n':
      this->NextWindow(view);
      break;

    case 'q':
      this->CloseWindow(view);
      break;
    }
}


void
MVControlApplicationState::FrameResized(MVControlView *view,
					float width, float height)
{
  if (view->mIgnoreResize)
    view->mIgnoreResize = false;
  else
    {
      if (! (modifiers() & B_CONTROL_KEY))
	view->ChangeMagnification
	  (min(width / view->mBitmap->Bounds().Width(),
	       height / view->mBitmap->Bounds().Height()));
      view->InscribedToFrame();
      view->Invalidate();
    }
}


void
MVControlApplicationState::MouseDown(MVControlView *view, BPoint point)
{
  int32 buttons;

  buttons = view->Window()->CurrentMessage()->FindInt32("buttons");

  if (buttons & B_SECONDARY_MOUSE_BUTTON)
    {
      point = view->ConvertToScreen(point);
      BRect clickToOpenRect = BRect(-5, -5, 5, 5).OffsetBySelf(point);
      BPopUpMenu *menu = new MVPopUpMenu(view);
      menu->SetAsyncAutoDestruct(true);
      menu->Go(point, true, false, clickToOpenRect, true);
    }
  else
    MVControlState::MouseDown(view, point);
}


void
MVControlApplicationState::MessageReceived(MVControlView *view,
					   BMessage *msg)
{
  switch (msg->what)
    {
    case MV_ZOOM_IN:
      this->ZoomIn(view);
      break;

    case MV_ZOOM_IN_WITHOUT_RESIZE:
      this->ZoomIn(view, B_CONTROL_KEY);
      break;

    case MV_ZOOM_OUT:
      this->ZoomOut(view);
      break;

    case MV_ZOOM_OUT_WITHOUT_RESIZE:
      this->ZoomOut(view, B_CONTROL_KEY);
      break;

    case MV_NORMALIZE:
      this->Normalize(view);
      break;

    case MV_FIT_TO_SCREEN:
      this->FitToScreen(view);
      break;

    case MV_ACTIVATE_WINDOW:
      this->ActivateWindow(view, msg->FindInt32("index"));
      break;

    case MV_PREV_WINDOW:
      this->PreviousWindow(view);
      break;

    case MV_NEXT_WINDOW:
      this->NextWindow(view);
      break;

    case MV_CLOSE_WINDOW:
      this->CloseWindow(view);
      break;

    case MV_QUIT_APPLICATION:
      this->QuitApplication(view);
      break;
 
    default:
      MVControlState::MessageReceived(view, msg);
      break;
    }
}


void
MVControlApplicationState::ZoomIn(MVControlView *view, int32 modifiers)
{
  view->ChangeMagnification(view->mMagnification * MV_KEY_MAGNIFICATION);
  if (! (modifiers & B_CONTROL_KEY))
    view->FitToScreen();
  view->InscribedToFrame();
  view->Invalidate();
}


void
MVControlApplicationState::ZoomOut(MVControlView *view, int32 modifiers)
{
  view->ChangeMagnification(view->mMagnification / MV_KEY_MAGNIFICATION);
  if (! (modifiers & B_CONTROL_KEY))
    view->FitToScreen();
  view->InscribedToFrame();
  view->Invalidate();
}


void
MVControlApplicationState::Normalize(MVControlView *view)
{
  view->ChangeMagnification(1.0);
  view->FitToScreen();
  view->InscribedToFrame();
}


void
MVControlApplicationState::FitToScreen(MVControlView *view)
{
  BRect screenFrame(BScreen().Frame().InsetByCopy(1, 1));
  
  view->ChangeMagnification
    (min(screenFrame.Width() / view->mBitmap->Bounds().Width(),
	 screenFrame.Height() / view->mBitmap->Bounds().Height()));
  view->FitToScreen();
  view->InscribedToFrame();
}


void
MVControlApplicationState::ActivateWindow(MVControlView *view, int index)
{
  BMessage msg(MV_ACTIVATE_WINDOW);
  msg.AddInt32("index", index);
  be_app->PostMessage(&msg);
}


void
MVControlApplicationState::PreviousWindow(MVControlView *view)
{
  be_app->PostMessage(MV_PREV_WINDOW);
}


void
MVControlApplicationState::NextWindow(MVControlView *view)
{
  be_app->PostMessage(MV_NEXT_WINDOW);
}


void
MVControlApplicationState::CloseWindow(MVControlView *view)
{
  view->Looper()->PostMessage(B_QUIT_REQUESTED);
}


void
MVControlApplicationState::QuitApplication(MVControlView *view)
{
  be_app->PostMessage(B_QUIT_REQUESTED);
}


