//
//  'MonoView' image/video wallpaper for BeOS.
//  Copyright (C) 1999, 2000
//  Okada Jun (yun@be-in.org)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  MonoView is yet another wallpaper running on BeOS Intel and PPC,
//  originally designed and developed by Okada Jun in 1999.
//  http://www.be-in.org/~yun/
//


#include <algorithm>
#include <strstream>

#include <Bitmap.h>
#include <File.h>
#include <Screen.h>
#include <Region.h>
#include <NodeMonitor.h>
#include <Window.h>
#include <Application.h>
#include <WindowScreen.h>

#include "MVDefs.h"
#include "MVControlState.h"
#include "MVControlApplicationState.h"
#include "MVControlReplicantState.h"
#include "MVControlView.h"


MVControlView::MVControlView(const entry_ref &ref)
  : MVReplicantableView(BRect(0, 0, 7, 7),
			"",
			B_FOLLOW_ALL_SIDES,
			B_WILL_DRAW | B_NAVIGABLE | B_FRAME_EVENTS |
			B_FULL_UPDATE_ON_RESIZE)
  , mRef(ref)
  , mBitmap(NULL)
  , mMagnification(1.0)
  , mOffset(0, 0)
  , mIgnoreResize(false)
  , mIsMouseHolding(false)
{
  this->SetViewColor(B_TRANSPARENT_32_BIT);
  mState = MVControlApplicationState::Instance();
}


MVControlView::MVControlView(BMessage *archive)
  : MVReplicantableView(archive)
  , mBitmap(NULL)
  , mIsMouseHolding(false)
{
  this->SetViewColor(B_TRANSPARENT_32_BIT);
  mMagnification = archive->FindFloat("MV:magnification");
  mOffset = archive->FindPoint("MV:offset");
  archive->FindRef("MV:ref", &mRef);

  mState = MVControlReplicantState::Instance();
}


MVControlView::~MVControlView(void)
{
}


status_t
MVControlView::Archive(BMessage *archive, bool deep) const
{
  if (MVReplicantableView::Archive(archive, deep) != B_OK)
    return B_ERROR;

  archive->AddFloat("MV:magnification", mMagnification);
  archive->AddPoint("MV:offset", mOffset);
  archive->AddRef("MV:ref", &mRef);

  return B_OK;
}


const char *
MVControlView::FileName(void) const
{
  return mRef.name;
}


void
MVControlView::Draw(BRect updateRect)
{
  mState->Draw(this, updateRect);
}


void
MVControlView::AllAttached(void)
{
  this->StartNodeMonitoring();

  mState->AllAttached(this);
}
			    

void
MVControlView::AllDetached(void)
{
  this->EndNodeMonitoring();

  mState->AllDetached(this);
}


void
MVControlView::KeyDown(const char *bytes, int32 numBytes)
{
  mState->KeyDown(this, bytes, numBytes);
}


void
MVControlView::MouseDown(BPoint point)
{
  mState->MouseDown(this, point);
}


void
MVControlView::MouseMoved(BPoint point, uint32 transit, const BMessage *msg)
{
  mState->MouseMoved(this, point, transit, msg);
}


void
MVControlView::MouseUp(BPoint point)
{
  mState->MouseUp(this, point);
}


void
MVControlView::FrameResized(float width, float height)
{
  mState->FrameResized(this, width, height);
}


void
MVControlView:: MessageReceived(BMessage *msg)
{
  switch (msg->what)
    {
    case MV_IGNORE_NEXT_RESIZE:
      mIgnoreResize = true;
      break;

    case B_NODE_MONITOR:
      this->PursuitNode(msg);
      break;

    default:
      mState->MessageReceived(this, msg);
    }
}


void
MVControlView::ChangeMagnification(float magnification)
{
  mOffset.x = int(this->Bounds().Width() / 2 + 
		 (mOffset.x - this->Bounds().Width() / 2)
		 * magnification / mMagnification);
  mOffset.y = int(this->Bounds().Height() / 2 + 
		 (mOffset.y - this->Bounds().Height() / 2)
		 * magnification / mMagnification);
  mMagnification = magnification;
  this->SetTitle();
}


void
MVControlView::InscribedToFrame(void)
{
  float bmpWidth(int(mBitmap->Bounds().Width() * mMagnification)),
    bmpHeight(int(mBitmap->Bounds().Height() * mMagnification));

  if (bmpWidth < this->Bounds().Width())
    mOffset.x = int((this->Bounds().Width() - bmpWidth) / 2);
  else if (mOffset.x > 0)
    mOffset.x = 0;
  else if (mOffset.x + bmpWidth < this->Bounds().Width())
    mOffset.x = this->Bounds().Width() - bmpWidth;
  
  if (bmpHeight < this->Bounds().Height())
    mOffset.y = int((this->Bounds().Height() - bmpHeight) / 2);
  else if (mOffset.y > 0)
    mOffset.y = 0;
  else if (mOffset.y + bmpHeight < this->Bounds().Height())
    mOffset.y = this->Bounds().Height() - bmpHeight;
    
}


void
MVControlView::FitToScreen(void)
{
  BRect screenFrame(BScreen().Frame().InsetByCopy(1, 1));
  float bmpWidth(int(mBitmap->Bounds().Width() * mMagnification)),
    bmpHeight(int(mBitmap->Bounds().Height() * mMagnification));
  float left, top, width, height;
  float minWidth, maxWidth, minHeight, maxHeight;
  
  this->Window()->GetSizeLimits(&minWidth, &maxWidth, &minHeight, &maxHeight);
  width = max(minWidth, min(bmpWidth, screenFrame.Width()));
  height = max(minHeight, min(bmpHeight, screenFrame.Height()));

  left = int(this->Window()->Frame().left +
	     (this->Frame().Width() - width) / 2);
  top = int(this->Window()->Frame().top +
	    (this->Frame().Height() - height) / 2);

  if (left < screenFrame.left)
    left = screenFrame.left;
  else if (left + width > screenFrame.right)
    left = screenFrame.right - width;

  if (top < screenFrame.top)
    top = screenFrame.top;
  else if (top + height > screenFrame.bottom)
    top = screenFrame.bottom - height;

  BRect targetFrame(left, top, left + width, top + height);
  if (targetFrame == this->Window()->Frame())
    {
      this->Invalidate();
      return;
    }

  if (::focus_follows_mouse()
      && this->Frame().Width() > width)
    {
      BPoint point(left + width / 2, top + height / 2);
      ::set_mouse_position(int32(point.x), int32(point.y));
    }

  be_app->ObscureCursor();

  this->Window()->PostMessage(MV_IGNORE_NEXT_RESIZE, this);
  this->Window()->MoveTo(left, top);
  this->Window()->ResizeTo(width, height);
}


void
MVControlView::SetTitle(void)
{
   if (this->IsReplicant()
      || ! this->Window())
    return;

   ostrstream buf;
  buf << mRef.name << "   " << int(mMagnification * 100) << '%' << '\0';
  this->Window()->SetTitle(buf.str());
}


void
MVControlView::StartNodeMonitoring(void)
{
  BEntry entry(&mRef);
  
  if (entry.InitCheck() == B_OK)
    {
      node_ref nref;
      entry.GetNodeRef(&nref);
      ::watch_node(&nref, B_ENTRY_MOVED, this);
    }
}


void
MVControlView::EndNodeMonitoring(void)
{
  ::stop_watching(this);
}


void
MVControlView::PursuitNode(const BMessage *msg)
{
  ino_t dir;
  dev_t dev;
  const char *name;
  
  msg->FindInt64("to directory", &dir);
  msg->FindInt32("device", &dev);
  name = msg->FindString("name");
  
  mRef = entry_ref(dev, dir, name);
}

