//
//  'MonoView' image/video wallpaper for BeOS.
//  Copyright (C) 1999, 2000
//  Okada Jun (yun@be-in.org)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  MonoView is yet another wallpaper running on BeOS Intel and PPC,
//  originally designed and developed by Okada Jun in 1999.
//  http://www.be-in.org/~yun/
//


#include <Looper.h>
#include <Alert.h>
#include <Dragger.h>
#include <Menu.h>
#include <PopUpMenu.h>
#include <MenuItem.h>
#include <Roster.h>
#include <Region.h>
#include <Application.h>

#include "MVDefs.h"
#include "MVForwardFilter.h"
#include "MVReplicantInputFilter.h"
#include "MVWorkspaceMenu.h"
#include "MVReplicantableView.h"


MVReplicantableView::MVReplicantableView(BRect frame, 
				     const char *name,
				     uint32 resizingMode,
				     uint32 flags)
  : BView(frame, name, resizingMode, flags)
  , mIsReplicant(false)
  , mLock(true)
  , mOnTracker(false)
  , mWorkspaceFilter(NULL)
{
  mDragger = new BDragger(BRect(0, 0, 7, 7), this); 
  this->AddChild(mDragger);

  this->SetMenu(mDragger->PopUp());
}


MVReplicantableView::MVReplicantableView(BMessage *archive)
  : BView(archive)
  , mIsReplicant(true)
  , mDragger(NULL)
{
  app_info info;
  const char *about;
  mOnTracker =  (be_app->GetAppInfo(&info) == B_OK &&
		 strcmp(info.signature, MV_TRACKER_SIGNATURE) == 0);

  archive->FindBool("MV:lock", &mLock);
  if (archive->FindString("MV:about", &about) == B_OK)
    mAboutContents = about;
  
  this->SetResizingMode(B_FOLLOW_NONE);
  mDragger = dynamic_cast<BDragger*>(this->FindView("_dragger_"));
  while (mDragger->IsHidden())
    mDragger->Show();
  mWorkspaceMenu = dynamic_cast<MVWorkspaceMenu*>
    (mDragger->PopUp()->FindItem("Workspaces")->Submenu());
  mWorkspaceMenu->SetTarget(this);

  mWorkspaceFilter = new MVForwardFilter(B_WORKSPACE_ACTIVATED, this);
  this->AddFilter(new MVReplicantInputFilter(this));
 
  this->WorkspaceActivated();
}


MVReplicantableView::~MVReplicantableView(void)
{
  if (mIsReplicant)
    delete mWorkspaceFilter;
}


status_t
MVReplicantableView::Archive(BMessage *archive, bool deep) const
{
  if (BView::Archive(archive, deep) != B_OK)
    return B_ERROR;

  archive->AddString("add_on", MV_SIGNATURE);
  archive->AddBool("MV:lock", mLock);
  archive->AddString("MV:about", mAboutContents.c_str());

  return B_OK;
}


bool
MVReplicantableView::IsReplicant(void) const
{
  return mIsReplicant;
}


bool
MVReplicantableView::IsOnTracker(void) const
{
  return mOnTracker;
}


bool
MVReplicantableView::IsLocked(void) const
{
  return mLock;
}


void
MVReplicantableView::SetAboutContents(const string &about)
{
  mAboutContents = about;
}


void
MVReplicantableView::SetMenu(BMenu *menu)
{
  BMenuItem *item;

  menu->AddItem(new BSeparatorItem(), 0);
  item = new BMenuItem(new MVWorkspaceMenu("Workspaces"));
  menu->AddItem(item, 0);
  item = new BMenuItem("Lock", new BMessage(MV_TOGGLE_LOCK));
  item->SetMarked(mLock);
  menu->AddItem(item, 0);
}


void
MVReplicantableView::AllAttached(void)
{
  if (mIsReplicant)
    {
      mDragger->PopUp()->SetTargetForItems(this);

      this->Window()->AddFilter(mWorkspaceFilter);

      if (mOnTracker)
	{
	  this->Parent()->SetFlags(this->Parent()->Flags() | B_DRAW_ON_CHILDREN);
	  this->Hide();
	  this->Show();
	}
   }
}


void
MVReplicantableView::AllDetached(void)
{
  if (mIsReplicant)
    {
      this->Window()->RemoveFilter(mWorkspaceFilter);
    }
}


void
MVReplicantableView::WorkspaceActivated(void)
{
  bool toShow(mWorkspaceMenu->Workspace() == B_ALL_WORKSPACES ||
	      mWorkspaceMenu->Workspace() == current_workspace());
  
  if (toShow && this->IsHidden())
    this->Show();

  if (! toShow && ! this->IsHidden())
    this->Hide();
}


void
MVReplicantableView::Draw(BRect updateRect)
{
  if (mOnTracker && mLock)
    {
      BPoint offset = this->ConvertToParent(BPoint(0, 0));
      BRegion region;
      this->GetClippingRegion(&region);
      region.OffsetBy(offset.x, offset.y);
      this->Parent()->ConstrainClippingRegion(&region);
      this->Parent()->Draw(this->ConvertToParent(updateRect));
      this->Parent()->ConstrainClippingRegion(NULL);
    }
}


void
MVReplicantableView::MessageReceived(BMessage *msg)
{
  switch (msg->what)
    {
    case B_ABOUT_REQUESTED:
      {
	BAlert *aboutDialog = 
	  new BAlert("About", mAboutContents.c_str(), "Ok");
	aboutDialog->Go();
      }

    case MV_SET_WORKSPACE:
    case B_WORKSPACE_ACTIVATED:
      {
	this->WorkspaceActivated();
	return;
      }

    case MV_TOGGLE_LOCK:
      {
	mLock = ! mLock;
	BMenuItem *item = mDragger->PopUp()->FindItem("Lock");
	item->SetMarked(mLock);
	this->Invalidate();
	return;
      }
      
    default:
      break;
    }

  BView::MessageReceived(msg);
}



