//
//  'MonoView' image/video wallpaper for BeOS.
//  Copyright (C) 1999, 2000
//  Okada Jun (yun@be-in.org)
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
//
//  MonoView is yet another wallpaper running on BeOS Intel and PPC,
//  originally designed and developed by Okada Jun in 1999.
//  http://www.be-in.org/~yun/
//


#include <cstring>
#include <algorithm>

#include <Bitmap.h>
#include <Region.h>
#include <Window.h>
#include <MessageQueue.h>
#include <MessageRunner.h>
#include <MediaFile.h>
#include <MediaTrack.h>

#include "MVDefs.h"
#include "MVControlNullState.h"
#include "MVControlReplicantState.h"
#include "MVVideoView.h"
#include "MVControlView.h"


enum
{
  PLAYBACK_NOTIFY = 'Pbnt',
};


MVVideoView::MVVideoView(const entry_ref &ref)
  : MVControlView(ref)
  , mPlaybackThread(B_ERROR)
  , mActiveBuffer(0)
  , mBackBuffer(NULL)
{
  this->Init();
  if (! mErr)
    this->ResizeTo(mBitmap->Bounds().Width(), mBitmap->Bounds().Height());
}


MVVideoView::MVVideoView(BMessage *archive)
  : MVControlView(archive)
  , mActiveBuffer(0)
  , mBackBuffer(NULL)
{
  this->Init();
  if (mErr)
    mState = MVControlNullState::Instance();
}


MVVideoView::~MVVideoView(void)
{
  if (mBackBuffer)
    delete mBackBuffer;
  if (mTrack)
    mFile->ReleaseTrack(mTrack);
  if (mFile)
    delete mFile;
}


BArchivable *
MVVideoView::Instantiate(BMessage *archive)
{
  if (validate_instantiation(archive, "MVVideoView"))
    return new MVVideoView(archive);

  return NULL;

}


status_t
MVVideoView::Archive(BMessage *msg, bool deep) const
{
  msg->AddString("add_on", MV_SIGNATURE);
    
  return MVControlView::Archive(msg, deep);
}


status_t
MVVideoView::InitCheck(void)
{
  return mErr;
}


void
MVVideoView::AllAttached(void)
{
  MVControlView::AllAttached();
  if (! mErr)
    this->Start();
  else if (this->IsReplicant())
    {
      BMessage *msg = new BMessage(PLAYBACK_NOTIFY);
      mPlaybackNotifier = 
	new BMessageRunner(BMessenger(this), msg, 1000000 * 5, 10);
    }
}


void
MVVideoView::AllDetached(void)
{
  if (! mErr)
    this->Stop();
  MVControlView::AllDetached();
}


void
MVVideoView::MessageReceived(BMessage *msg)
{
  switch (msg->what)
    {
    default:
      MVControlView::MessageReceived(msg);
      break;

    case PLAYBACK_NOTIFY:
      this->Init();
      if (mErr != B_OK)
	break;
      BMessageQueue *queue = this->Window()->MessageQueue();
      while ((msg = queue->FindMessage(uint32(PLAYBACK_NOTIFY))))
	{
	  queue->RemoveMessage(msg);
	  delete msg;
	}
      delete mPlaybackNotifier;
      mState = MVControlReplicantState::Instance();
      this->Start();
      break;
    }
}


void
MVVideoView::Hide(void)
{
  MVControlView::Hide();
  if (mPlaybackThread != B_ERROR)
    ::suspend_thread(mPlaybackThread);
}


void
MVVideoView::Show(void)
{
  MVControlView::Show();
  if (mPlaybackThread != B_ERROR
      && ! this->IsHidden())
    ::resume_thread(mPlaybackThread);
}


void
MVVideoView::Init(void)
{
  media_format format;

  mFile = NULL;
  mTrack = NULL;
  mIsPlaying = false;

  mFile = new BMediaFile(&mRef);
  if ((mErr = mFile->InitCheck()))
    return;

  for (int i = 0; i < mFile->CountTracks(); i++)
    {
      if ((mTrack = mFile->TrackAt(i)) != NULL &&
	  (mErr = mTrack->EncodedFormat(&format)) == B_OK &&
	  format.type == B_MEDIA_ENCODED_VIDEO)
	break;
      mFile->ReleaseTrack(mTrack);
      mTrack = NULL;
    }

  if (! mTrack)
    {
      mErr = B_ERROR;
      return;
    }

  BRect bounds(0.0, 0.0,
	       format.u.encoded_video.output.display.line_width - 1.0,
	       format.u.encoded_video.output.display.line_count - 1.0);
  mBackBuffer = new BBitmap(bounds, B_RGB32);

  memset(&format, 0, sizeof(media_format));
  format.u.raw_video.last_active = (int32) (bounds.Height() - 1.0);
  format.u.raw_video.orientation = B_VIDEO_TOP_LEFT_RIGHT;
  format.u.raw_video.pixel_width_aspect = 1;
  format.u.raw_video.pixel_height_aspect = 3;
  format.u.raw_video.display.format = mBackBuffer->ColorSpace();
  format.u.raw_video.display.line_width = (int32) bounds.Width();
  format.u.raw_video.display.line_count = (int32) bounds.Height();
  format.u.raw_video.display.bytes_per_row = mBackBuffer->BytesPerRow();
  mErr = mTrack->DecodedFormat(&format);
  if (mErr != B_OK)
    {
      delete mBackBuffer;
      mBackBuffer = NULL;
      return;
    }

  mBitmap = mBackBuffer;

  mErr = B_OK;
};


void
MVVideoView::Start(void)
{
  mIsPlaying = true;
  
  mPlaybackThread = spawn_thread(_PlaybackLoop,
				 mRef.name, 
				 B_LOW_PRIORITY,
				 this);
  resume_thread(mPlaybackThread);
}


void
MVVideoView::Stop(void)
{
  kill_thread(mPlaybackThread);
}


int32
MVVideoView::_PlaybackLoop(void *data)
{
  static_cast<MVVideoView*>(data)->PlaybackLoop();
  return 0;
}


void
MVVideoView::PlaybackLoop(void)
{
  bigtime_t wait, previousFrameTime;

  wait = ::system_time();
  previousFrameTime = 0;

   while (true)
    {
      int64 frameCount;
      status_t err;

      err = mTrack->ReadFrames(static_cast<char*>(mBackBuffer->Bits()), 
			       &frameCount, NULL);
      if (err != B_OK)
	{
	  int64 startFrame(0);
	  mTrack->SeekToFrame(&startFrame);
	  previousFrameTime = 0;
	  continue;
	}

      ::snooze_until(wait, B_SYSTEM_TIMEBASE);
      wait = ::system_time() + mTrack->CurrentTime() - previousFrameTime;
      previousFrameTime = mTrack->CurrentTime();
      this->LockLooper();
      this->Update();
      this->UnlockLooper();
    }
}


void
MVVideoView::Update(void)
{
  this->Window()->BeginViewTransaction();
  float y;
  BRegion region;
  BRect frame = this->Frame().OffsetToSelf(0, 0);
  for (y = 0; y <= frame.bottom; y += 200)
    {
      BRect drawFrame(frame.left, y,
		      frame.right, min(y + 199, frame.bottom));
      region.Set(drawFrame);
      this->ConstrainClippingRegion(&region);
      this->Draw(drawFrame);
    }
  this->ConstrainClippingRegion(NULL);
  this->Window()->EndViewTransaction();
}
